@@ -10,6 +10,7 @@ from oort.shared.config import get_oort_config_upload_folder_sections, get_oort_
 from oort.shared.constants import OORT_UPLOADER_FOLDER_DETECTION_TICK_SECONDS
 from oort.shared.identity import Identity
 from oort.shared.models import Upload
+from oort.shared.utils import is_hidden
 from . import eventhandler
 
 
@@ -39,13 +40,21 @@ class PathsObserver(Observer):
 
     def _detect_watched_folders(self):
         mapping_keys = self._mapping.keys()
+        folder_sections = get_oort_config_upload_folder_sections()
 
-        for folder_section in get_oort_config_upload_folder_sections():
+        # Unschedule all paths that are not observed anymore.
+        folder_section_paths = [section.get('path') for section in folder_sections]
+        for mapping_key in mapping_keys:
+            if mapping_key not in folder_section_paths:
+                self.unschedule(mapping_key)
+
+        # Now, schedule, if not yet done, all folders to be watched.
+        for folder_section in folder_sections:
             folder_path = folder_section.get('path')
             identity = Identity.from_folder_section(folder_section)
 
             if folder_path in mapping_keys:
-                # Folder is already watched. We need to update its config only if identity is differnt.
+                # Folder is already watched. We need to update its config only if identity is different.
                 handler = self._mapping.get(folder_path).get('handler')
                 if handler is not None and identity != handler.identity:
                     self._unschedule_watch(folder_path)
@@ -53,6 +62,7 @@ class PathsObserver(Observer):
             else:
                 self._schedule_watch(folder_path, identity, initial_walk=True)
 
+        # Every OORT_UPLOADER_FOLDER_DETECTION_TICK_SECONDS seconds, new folders are being checked.
         threading.Timer(OORT_UPLOADER_FOLDER_DETECTION_TICK_SECONDS, self._detect_watched_folders).start()
 
     def _unschedule_watch(self, folder_path: str) -> None:
@@ -71,8 +81,11 @@ class PathsObserver(Observer):
         watcher = self.schedule(event_handler, folder_path, recursive=True)
         self._mapping[folder_path] = {'watcher': watcher, 'handler': event_handler}
 
-        if initial_walk is True:
-            threading.Thread(target=self._start_initial_walk, args=(folder_path, event_handler)).start()
+        # Perform collect-only quick initial walk in a synchronous way...
+        self._start_initial_walk(folder_path, event_handler)
+
+        # ...then launch the upload loop.
+        event_handler.launch_restart_loop()
 
     def _start_initial_walk(self, folder_path, event_handler):
         self._logger.info(f'{self.log_prefix} Starting initial walk inside folder {folder_path}...')
@@ -85,7 +98,7 @@ class PathsObserver(Observer):
         file_count, event_count, ignore_count = 0, 0, 0
         for path in root_path.glob('**/*'):
             # Skipping both hidden files and hidden directories.
-            if any([part for part in path.parts if len(part) > 0 and part[0] == '.']) or not path.is_file():
+            if is_hidden(path) or not path.is_file():
                 continue
 
             file_count += 1
@@ -96,6 +109,9 @@ class PathsObserver(Observer):
                     self._logger.info(msg)
             else:
                 event_count += 1
+                # The dispatch of this event will trigger the collect of basic info about the file, and create
+                # an entry in the Upload DB. No zip, no upload, no preparation whatsoever. This is handled by
+                # the event_handler's "restart_uploads" loop.
                 event = FileCreatedEvent(str(path))
                 event_handler.dispatch(event)
             time.sleep(0.01)
@@ -107,8 +123,6 @@ class PathsObserver(Observer):
         msg += f'dispatched {event_count} events for {file_count} files.'
         self._logger.info(msg)
 
-        event_handler.launch_restart_loop()
-
     def forget_folder(self, folder_path: str) -> None:
         if folder_path in self._mapping.keys():
             self._logger.info(f'{self.log_prefix} Forgetting to observe folder {folder_path}')
